@@ -14,26 +14,39 @@
 
 """Conformance tests for retry. Verifies correct behavior around retryable errors, idempotency and preconditions."""
 
-import os
-import requests
-import tempfile
-import uuid
-import logging
 import functools
+import logging
+import os
+import subprocess
+import tempfile
+import time
+import uuid
+
 import pytest
+import requests
+
+from six.moves.urllib import parse as urlparse
 
-from google.cloud import storage
 from google.auth.credentials import AnonymousCredentials
+from google.cloud import storage
+from google.cloud.exceptions import NotFound
+from google.cloud.storage.hmac_key import HMACKeyMetadata
 
 from . import _read_local_json
 
 
-_CONFORMANCE_TESTS = _read_local_json("retry_strategy_test_data.json")[
-    "retryStrategyTests"
-]
+_CONFORMANCE_TESTS = _read_local_json("retry_strategy_test_data.json")["retryTests"]
 
-_STORAGE_EMULATOR_ENV_VAR = "STORAGE_EMULATOR_HOST"
-"""Environment variable defining host for Storage testbench emulator."""
+"""Environment variable or default host for Storage testbench emulator."""
+_HOST = os.environ.get("STORAGE_EMULATOR_HOST", "http://localhost:9000")
+_PORT = urlparse.urlsplit(_HOST).port
+
+"""The storage testbench docker image info and commands."""
+_DEFAULT_IMAGE_NAME = "gcr.io/cloud-devrel-public-resources/storage-testbench"
+_DEFAULT_IMAGE_TAG = "latest"
+_DOCKER_IMAGE = "{}:{}".format(_DEFAULT_IMAGE_NAME, _DEFAULT_IMAGE_TAG)
+_PULL_CMD = ["docker", "pull", _DOCKER_IMAGE]
+_RUN_CMD = ["docker", "run", "--rm", "-d", "-p", "{}:9000".format(_PORT), _DOCKER_IMAGE]
 
 _CONF_TEST_PROJECT_ID = "my-project-id"
 _CONF_TEST_SERVICE_ACCOUNT_EMAIL = (
@@ -171,8 +184,12 @@ def bucket_lock_retention_policy(client, _preconditions, **resources):
     bucket.lock_retention_policy()
 
 
+def client_get_service_account_email(client, _preconditions, **_):
+    client.get_service_account_email()
+
+
 def notification_create(client, _preconditions, **resources):
-    bucket = client.get_bucket(resources.get("bucket").name)
+    bucket = client.bucket(resources.get("bucket").name)
     notification = bucket.notification()
     notification.create()
 
@@ -217,13 +234,49 @@ def client_list_hmac_keys(client, _preconditions, **_):
         pass
 
 
-def client_get_service_account_email(client, _preconditions, **_):
-    client.get_service_account_email()
+def client_get_hmac_key_metadata(client, _preconditions, **resources):
+    access_id = resources.get("hmac_key").access_id
+    client.get_hmac_key_metadata(access_id=access_id)
+
+
+def hmac_key_exists(client, _preconditions, **resources):
+    access_id = resources.get("hmac_key").access_id
+    hmac_key = HMACKeyMetadata(client, access_id=access_id)
+    hmac_key.exists()
+
+
+def hmac_key_reload(client, _preconditions, **resources):
+    access_id = resources.get("hmac_key").access_id
+    hmac_key = HMACKeyMetadata(client, access_id=access_id)
+    hmac_key.reload()
+
+
+def hmac_key_delete(client, _preconditions, **resources):
+    access_id = resources.get("hmac_key").access_id
+    hmac_key = HMACKeyMetadata(client, access_id=access_id)
+    hmac_key.state = "INACTIVE"
+    hmac_key.update()
+    hmac_key.delete()
+
+
+def client_create_hmac_key(client, _preconditions, **_):
+    client.create_hmac_key(service_account_email=_CONF_TEST_SERVICE_ACCOUNT_EMAIL)
+
+
+def hmac_key_update(client, _preconditions, **resources):
+    access_id = resources.get("hmac_key").access_id
+    etag = resources.get("hmac_key").etag
+    hmac_key = HMACKeyMetadata(client, access_id=access_id)
+    if _preconditions:
+        pytest.skip("Etag is not yet supported")
+        hmac_key.etag = etag
+    hmac_key.state = "INACTIVE"
+    hmac_key.update()
 
 
 def bucket_patch(client, _preconditions, **resources):
-    bucket = client.get_bucket(resources.get("bucket").name)
-    metageneration = bucket.metageneration
+    bucket = client.bucket(resources.get("bucket").name)
+    metageneration = resources.get("bucket").metageneration
     bucket.storage_class = "COLDLINE"
     if _preconditions:
         bucket.patch(if_metageneration_match=metageneration)
@@ -232,8 +285,8 @@ def bucket_patch(client, _preconditions, **resources):
 
 
 def bucket_update(client, _preconditions, **resources):
-    bucket = client.get_bucket(resources.get("bucket").name)
-    metageneration = bucket.metageneration
+    bucket = client.bucket(resources.get("bucket").name)
+    metageneration = resources.get("bucket").metageneration
     bucket._properties = {"storageClass": "STANDARD"}
     if _preconditions:
         bucket.update(if_metageneration_match=metageneration)
@@ -242,7 +295,7 @@ def bucket_update(client, _preconditions, **resources):
 
 
 def bucket_set_iam_policy(client, _preconditions, **resources):
-    bucket = client.get_bucket(resources.get("bucket").name)
+    bucket = client.bucket(resources.get("bucket").name)
     role = "roles/storage.objectViewer"
     member = _CONF_TEST_SERVICE_ACCOUNT_EMAIL
 
@@ -423,6 +476,156 @@ def blob_create_resumable_upload_session(client, _preconditions, **resources):
         blob.create_resumable_upload_session()
 
 
+def blob_make_private(client, _preconditions, **resources):
+    bucket = resources.get("bucket")
+    object = resources.get("object")
+    blob = client.bucket(bucket.name).blob(object.name)
+    if _preconditions:
+        blob.make_private(if_metageneration_match=object.metageneration)
+    else:
+        blob.make_private()
+
+
+def blob_make_public(client, _preconditions, **resources):
+    bucket = resources.get("bucket")
+    object = resources.get("object")
+    blob = client.bucket(bucket.name).blob(object.name)
+    if _preconditions:
+        blob.make_public(if_metageneration_match=object.metageneration)
+    else:
+        blob.make_public()
+
+
+def bucket_make_private(client, _preconditions, **resources):
+    bucket = client.bucket(resources.get("bucket").name)
+    if _preconditions:
+        bucket.make_private(
+            if_metageneration_match=resources.get("bucket").metageneration
+        )
+    else:
+        bucket.make_private()
+
+
+def bucket_make_public(client, _preconditions, **resources):
+    bucket = client.bucket(resources.get("bucket").name)
+    if _preconditions:
+        bucket.make_public(
+            if_metageneration_match=resources.get("bucket").metageneration
+        )
+    else:
+        bucket.make_public()
+
+
+def bucket_acl_reload(client, _preconditions, **resources):
+    bucket = client.bucket(resources.get("bucket").name)
+    bucket.acl.reload()
+
+
+def bucket_acl_save(client, _preconditions, **resources):
+    bucket = client.bucket(resources.get("bucket").name)
+    bucket.acl.user(_CONF_TEST_SERVICE_ACCOUNT_EMAIL).grant_owner()
+    if _preconditions:
+        bucket.acl.save(if_metageneration_match=resources.get("bucket").metageneration)
+    else:
+        bucket.acl.save()
+
+
+def bucket_acl_save_predefined(client, _preconditions, **resources):
+    bucket = client.bucket(resources.get("bucket").name)
+    if _preconditions:
+        bucket.acl.save_predefined(
+            "bucketOwnerFullControl",
+            if_metageneration_match=resources.get("bucket").metageneration,
+        )
+    else:
+        bucket.acl.save_predefined("bucketOwnerFullControl")
+
+
+def bucket_acl_clear(client, _preconditions, **resources):
+    bucket = client.bucket(resources.get("bucket").name)
+    if _preconditions:
+        bucket.acl.clear(if_metageneration_match=resources.get("bucket").metageneration)
+    else:
+        bucket.acl.clear()
+
+
+def default_object_acl_reload(client, _preconditions, **resources):
+    bucket = client.bucket(resources.get("bucket").name)
+    bucket.default_object_acl.reload()
+
+
+def default_object_acl_save(client, _preconditions, **resources):
+    bucket = client.bucket(resources.get("bucket").name)
+    bucket.default_object_acl.user(_CONF_TEST_SERVICE_ACCOUNT_EMAIL).grant_owner()
+    if _preconditions:
+        bucket.default_object_acl.save(
+            if_metageneration_match=resources.get("bucket").metageneration
+        )
+    else:
+        bucket.default_object_acl.save()
+
+
+def default_object_acl_save_predefined(client, _preconditions, **resources):
+    bucket = client.bucket(resources.get("bucket").name)
+    if _preconditions:
+        bucket.default_object_acl.save_predefined(
+            "bucketOwnerFullControl",
+            if_metageneration_match=resources.get("bucket").metageneration,
+        )
+    else:
+        bucket.default_object_acl.save_predefined("bucketOwnerFullControl")
+
+
+def default_object_acl_clear(client, _preconditions, **resources):
+    bucket = client.bucket(resources.get("bucket").name)
+    if _preconditions:
+        bucket.default_object_acl.clear(
+            if_metageneration_match=resources.get("bucket").metageneration
+        )
+    else:
+        bucket.default_object_acl.clear()
+
+
+def object_acl_reload(client, _preconditions, **resources):
+    bucket = resources.get("bucket")
+    object = resources.get("object")
+    blob = client.bucket(bucket.name).blob(object.name)
+    blob.acl.reload()
+
+
+def object_acl_save(client, _preconditions, **resources):
+    bucket = resources.get("bucket")
+    object = resources.get("object")
+    blob = client.bucket(bucket.name).blob(object.name)
+    blob.acl.user(_CONF_TEST_SERVICE_ACCOUNT_EMAIL).grant_owner()
+    if _preconditions:
+        blob.acl.save(if_metageneration_match=object.metageneration)
+    else:
+        blob.acl.save()
+
+
+def object_acl_save_predefined(client, _preconditions, **resources):
+    bucket = resources.get("bucket")
+    object = resources.get("object")
+    blob = client.bucket(bucket.name).blob(object.name)
+    if _preconditions:
+        blob.acl.save_predefined(
+            "bucketOwnerFullControl", if_metageneration_match=object.metageneration
+        )
+    else:
+        blob.acl.save_predefined("bucketOwnerFullControl")
+
+
+def object_acl_clear(client, _preconditions, **resources):
+    bucket = resources.get("bucket")
+    object = resources.get("object")
+    blob = client.bucket(bucket.name).blob(object.name)
+    if _preconditions:
+        blob.acl.clear(if_metageneration_match=object.metageneration)
+    else:
+        blob.acl.clear()
+
+
 ########################################################################################################################################
 ### Method Invocation Mapping ##########################################################################################################
 ########################################################################################################################################
@@ -434,7 +637,8 @@ def blob_create_resumable_upload_session(client, _preconditions, **resources):
 # read or just a metadata get).
 
 method_mapping = {
-    "storage.buckets.delete": [bucket_delete],  # S1 start
+    "storage.bucket_acl.list": [bucket_acl_reload],  # S1 start
+    "storage.buckets.delete": [bucket_delete],
     "storage.buckets.get": [
         client_get_bucket,
         bucket_reload,
@@ -446,6 +650,14 @@ method_mapping = {
     "storage.buckets.list": [client_list_buckets],
     "storage.buckets.lockRetentionPolicy": [bucket_lock_retention_policy],
     "storage.buckets.testIamPermissions": [bucket_test_iam_permissions],
+    "storage.default_object_acl.list": [default_object_acl_reload],
+    "storage.hmacKey.delete": [hmac_key_delete],
+    "storage.hmacKey.get": [
+        client_get_hmac_key_metadata,
+        hmac_key_exists,
+        hmac_key_reload,
+    ],
+    "storage.hmacKey.list": [client_list_hmac_keys],
     "storage.notifications.delete": [notification_delete],
     "storage.notifications.get": [
         bucket_get_notification,
@@ -453,6 +665,7 @@ method_mapping = {
         notification_reload,
     ],
     "storage.notifications.list": [bucket_list_notifications],
+    "storage.object_acl.list": [object_acl_reload],
     "storage.objects.get": [
         bucket_get_blob,
         blob_exists,
@@ -462,14 +675,22 @@ method_mapping = {
         blob_download_as_text,
         blobreader_read,
     ],
-    "storage.objects.list": [
-        client_list_blobs,
-        bucket_list_blobs,
-        bucket_delete,
-    ],  # S1 end
-    "storage.buckets.patch": [bucket_patch],  # S2/S3 start
+    "storage.objects.list": [client_list_blobs, bucket_list_blobs, bucket_delete],
+    "storage.serviceaccount.get": [client_get_service_account_email],  # S1 end
+    "storage.buckets.patch": [
+        bucket_patch,
+        bucket_make_public,
+        bucket_make_private,
+        bucket_acl_save,
+        bucket_acl_save_predefined,
+        bucket_acl_clear,
+        default_object_acl_save,
+        default_object_acl_save_predefined,
+        default_object_acl_clear,
+    ],  # S2/S3 start
     "storage.buckets.setIamPolicy": [bucket_set_iam_policy],
     "storage.buckets.update": [bucket_update],
+    "storage.hmacKey.update": [hmac_key_update],
     "storage.objects.compose": [blob_compose],
     "storage.objects.copy": [bucket_copy_blob, bucket_rename_blob],
     "storage.objects.delete": [
@@ -485,9 +706,18 @@ method_mapping = {
         blobwriter_write,
         blob_create_resumable_upload_session,
     ],
-    "storage.objects.patch": [blob_patch],
+    "storage.objects.patch": [
+        blob_patch,
+        object_acl_save,
+        object_acl_save_predefined,
+        object_acl_clear,
+        blob_make_private,
+        blob_make_public,
+    ],
     "storage.objects.rewrite": [blob_rewrite, blob_update_storage_class],
     "storage.objects.update": [blob_update],  # S2/S3 end
+    "storage.hmacKey.create": [client_create_hmac_key],  # S4 start
+    "storage.notifications.insert": [notification_create],
 }
 
 
@@ -498,11 +728,10 @@ method_mapping = {
 
 @pytest.fixture
 def client():
-    host = os.environ.get(_STORAGE_EMULATOR_ENV_VAR)
     client = storage.Client(
         project=_CONF_TEST_PROJECT_ID,
         credentials=AnonymousCredentials(),
-        client_options={"api_endpoint": host},
+        client_options={"api_endpoint": _HOST},
     )
     return client
 
@@ -514,9 +743,7 @@ def bucket(client):
     yield bucket
     try:
         bucket.delete(force=True)
-    except Exception:
-        # in cases where resources are deleted within the test
-        # TODO(cathyo@): narrow except to NotFound once the emulator response issue is resolved
+    except NotFound:  # in cases where bucket is deleted within the test
         pass
 
 
@@ -528,7 +755,7 @@ def object(client, bucket):
     yield blob
     try:
         blob.delete()
-    except Exception:  # in cases where resources are deleted within the test
+    except NotFound:  # in cases where object is deleted within the test
         pass
 
 
@@ -540,7 +767,7 @@ def notification(client, bucket):
     yield notification
     try:
         notification.delete()
-    except Exception:  # in cases where resources are deleted within the test
+    except NotFound:  # in cases where notification is deleted within the test
         pass
 
 
@@ -555,7 +782,7 @@ def hmac_key(client):
         hmac_key.state = "INACTIVE"
         hmac_key.update()
         hmac_key.delete()
-    except Exception:  # in cases where resources are deleted within the test
+    except NotFound:  # in cases where hmac_key is deleted within the test
         pass
 
 
@@ -704,26 +931,30 @@ def run_test_case(
 ### Run Conformance Tests for Retry Strategy ###########################################################################################
 ########################################################################################################################################
 
-for scenario in _CONFORMANCE_TESTS:
-    host = os.environ.get(_STORAGE_EMULATOR_ENV_VAR)
-    if host is None:
-        logging.error(
-            "This test must use the testbench emulator; set STORAGE_EMULATOR_HOST to run."
-        )
-        break
+# Pull storage-testbench docker image
+subprocess.run(_PULL_CMD)
+time.sleep(5)
 
-    id = scenario["id"]
-    methods = scenario["methods"]
-    cases = scenario["cases"]
-    for c in cases:
-        for m in methods:
-            method_name = m["name"]
-            if method_name not in method_mapping:
-                logging.info("No tests for operation {}".format(method_name))
-                continue
+# Run docker image to start storage-testbench
+with subprocess.Popen(_RUN_CMD) as proc:
+    # Run retry conformance tests
+    for scenario in _CONFORMANCE_TESTS:
+        id = scenario["id"]
+        methods = scenario["methods"]
+        cases = scenario["cases"]
+        for i, c in enumerate(cases):
+            for m in methods:
+                method_name = m["name"]
+                if method_name not in method_mapping:
+                    logging.info("No tests for operation {}".format(method_name))
+                    continue
 
-            for lib_func in method_mapping[method_name]:
-                test_name = "test-S{}-{}-{}".format(id, method_name, lib_func.__name__)
-                globals()[test_name] = functools.partial(
-                    run_test_case, id, m, c, lib_func, host
-                )
+                for lib_func in method_mapping[method_name]:
+                    test_name = "test-S{}-{}-{}-{}".format(
+                        id, method_name, lib_func.__name__, i
+                    )
+                    globals()[test_name] = functools.partial(
+                        run_test_case, id, m, c, lib_func, _HOST
+                    )
+    time.sleep(5)
+    proc.kill()
